# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union

from google.api_core import gapic_v1  # type: ignore
from google.api_core import grpc_helpers_async  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
import packaging.version

import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.websecurityscanner_v1beta.types import finding
from google.cloud.websecurityscanner_v1beta.types import scan_config
from google.cloud.websecurityscanner_v1beta.types import scan_config as gcw_scan_config
from google.cloud.websecurityscanner_v1beta.types import scan_run
from google.cloud.websecurityscanner_v1beta.types import web_security_scanner
from google.protobuf import empty_pb2  # type: ignore
from .base import WebSecurityScannerTransport, DEFAULT_CLIENT_INFO
from .grpc import WebSecurityScannerGrpcTransport


class WebSecurityScannerGrpcAsyncIOTransport(WebSecurityScannerTransport):
    """gRPC AsyncIO backend transport for WebSecurityScanner.

    Cloud Web Security Scanner Service identifies security
    vulnerabilities in web applications hosted on Google Cloud
    Platform. It crawls your application, and attempts to exercise
    as many user inputs and event handlers as possible.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "websecurityscanner.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            **self_signed_jwt_kwargs,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "websecurityscanner.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: aio.Channel = None,
        api_mtls_endpoint: str = None,
        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
        ssl_channel_credentials: grpc.ChannelCredentials = None,
        client_cert_source_for_mtls: Callable[[], Tuple[bytes, bytes]] = None,
        quota_project_id=None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[aio.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or applicatin default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                credentials=self._credentials,
                credentials_file=credentials_file,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def create_scan_config(
        self,
    ) -> Callable[
        [web_security_scanner.CreateScanConfigRequest],
        Awaitable[gcw_scan_config.ScanConfig],
    ]:
        r"""Return a callable for the create scan config method over gRPC.

        Creates a new ScanConfig.

        Returns:
            Callable[[~.CreateScanConfigRequest],
                    Awaitable[~.ScanConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_scan_config" not in self._stubs:
            self._stubs["create_scan_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/CreateScanConfig",
                request_serializer=web_security_scanner.CreateScanConfigRequest.serialize,
                response_deserializer=gcw_scan_config.ScanConfig.deserialize,
            )
        return self._stubs["create_scan_config"]

    @property
    def delete_scan_config(
        self,
    ) -> Callable[
        [web_security_scanner.DeleteScanConfigRequest], Awaitable[empty_pb2.Empty]
    ]:
        r"""Return a callable for the delete scan config method over gRPC.

        Deletes an existing ScanConfig and its child
        resources.

        Returns:
            Callable[[~.DeleteScanConfigRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_scan_config" not in self._stubs:
            self._stubs["delete_scan_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/DeleteScanConfig",
                request_serializer=web_security_scanner.DeleteScanConfigRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_scan_config"]

    @property
    def get_scan_config(
        self,
    ) -> Callable[
        [web_security_scanner.GetScanConfigRequest], Awaitable[scan_config.ScanConfig]
    ]:
        r"""Return a callable for the get scan config method over gRPC.

        Gets a ScanConfig.

        Returns:
            Callable[[~.GetScanConfigRequest],
                    Awaitable[~.ScanConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_scan_config" not in self._stubs:
            self._stubs["get_scan_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/GetScanConfig",
                request_serializer=web_security_scanner.GetScanConfigRequest.serialize,
                response_deserializer=scan_config.ScanConfig.deserialize,
            )
        return self._stubs["get_scan_config"]

    @property
    def list_scan_configs(
        self,
    ) -> Callable[
        [web_security_scanner.ListScanConfigsRequest],
        Awaitable[web_security_scanner.ListScanConfigsResponse],
    ]:
        r"""Return a callable for the list scan configs method over gRPC.

        Lists ScanConfigs under a given project.

        Returns:
            Callable[[~.ListScanConfigsRequest],
                    Awaitable[~.ListScanConfigsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_scan_configs" not in self._stubs:
            self._stubs["list_scan_configs"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/ListScanConfigs",
                request_serializer=web_security_scanner.ListScanConfigsRequest.serialize,
                response_deserializer=web_security_scanner.ListScanConfigsResponse.deserialize,
            )
        return self._stubs["list_scan_configs"]

    @property
    def update_scan_config(
        self,
    ) -> Callable[
        [web_security_scanner.UpdateScanConfigRequest],
        Awaitable[gcw_scan_config.ScanConfig],
    ]:
        r"""Return a callable for the update scan config method over gRPC.

        Updates a ScanConfig. This method support partial
        update of a ScanConfig.

        Returns:
            Callable[[~.UpdateScanConfigRequest],
                    Awaitable[~.ScanConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_scan_config" not in self._stubs:
            self._stubs["update_scan_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/UpdateScanConfig",
                request_serializer=web_security_scanner.UpdateScanConfigRequest.serialize,
                response_deserializer=gcw_scan_config.ScanConfig.deserialize,
            )
        return self._stubs["update_scan_config"]

    @property
    def start_scan_run(
        self,
    ) -> Callable[
        [web_security_scanner.StartScanRunRequest], Awaitable[scan_run.ScanRun]
    ]:
        r"""Return a callable for the start scan run method over gRPC.

        Start a ScanRun according to the given ScanConfig.

        Returns:
            Callable[[~.StartScanRunRequest],
                    Awaitable[~.ScanRun]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "start_scan_run" not in self._stubs:
            self._stubs["start_scan_run"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/StartScanRun",
                request_serializer=web_security_scanner.StartScanRunRequest.serialize,
                response_deserializer=scan_run.ScanRun.deserialize,
            )
        return self._stubs["start_scan_run"]

    @property
    def get_scan_run(
        self,
    ) -> Callable[
        [web_security_scanner.GetScanRunRequest], Awaitable[scan_run.ScanRun]
    ]:
        r"""Return a callable for the get scan run method over gRPC.

        Gets a ScanRun.

        Returns:
            Callable[[~.GetScanRunRequest],
                    Awaitable[~.ScanRun]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_scan_run" not in self._stubs:
            self._stubs["get_scan_run"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/GetScanRun",
                request_serializer=web_security_scanner.GetScanRunRequest.serialize,
                response_deserializer=scan_run.ScanRun.deserialize,
            )
        return self._stubs["get_scan_run"]

    @property
    def list_scan_runs(
        self,
    ) -> Callable[
        [web_security_scanner.ListScanRunsRequest],
        Awaitable[web_security_scanner.ListScanRunsResponse],
    ]:
        r"""Return a callable for the list scan runs method over gRPC.

        Lists ScanRuns under a given ScanConfig, in
        descending order of ScanRun stop time.

        Returns:
            Callable[[~.ListScanRunsRequest],
                    Awaitable[~.ListScanRunsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_scan_runs" not in self._stubs:
            self._stubs["list_scan_runs"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/ListScanRuns",
                request_serializer=web_security_scanner.ListScanRunsRequest.serialize,
                response_deserializer=web_security_scanner.ListScanRunsResponse.deserialize,
            )
        return self._stubs["list_scan_runs"]

    @property
    def stop_scan_run(
        self,
    ) -> Callable[
        [web_security_scanner.StopScanRunRequest], Awaitable[scan_run.ScanRun]
    ]:
        r"""Return a callable for the stop scan run method over gRPC.

        Stops a ScanRun. The stopped ScanRun is returned.

        Returns:
            Callable[[~.StopScanRunRequest],
                    Awaitable[~.ScanRun]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "stop_scan_run" not in self._stubs:
            self._stubs["stop_scan_run"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/StopScanRun",
                request_serializer=web_security_scanner.StopScanRunRequest.serialize,
                response_deserializer=scan_run.ScanRun.deserialize,
            )
        return self._stubs["stop_scan_run"]

    @property
    def list_crawled_urls(
        self,
    ) -> Callable[
        [web_security_scanner.ListCrawledUrlsRequest],
        Awaitable[web_security_scanner.ListCrawledUrlsResponse],
    ]:
        r"""Return a callable for the list crawled urls method over gRPC.

        List CrawledUrls under a given ScanRun.

        Returns:
            Callable[[~.ListCrawledUrlsRequest],
                    Awaitable[~.ListCrawledUrlsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_crawled_urls" not in self._stubs:
            self._stubs["list_crawled_urls"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/ListCrawledUrls",
                request_serializer=web_security_scanner.ListCrawledUrlsRequest.serialize,
                response_deserializer=web_security_scanner.ListCrawledUrlsResponse.deserialize,
            )
        return self._stubs["list_crawled_urls"]

    @property
    def get_finding(
        self,
    ) -> Callable[[web_security_scanner.GetFindingRequest], Awaitable[finding.Finding]]:
        r"""Return a callable for the get finding method over gRPC.

        Gets a Finding.

        Returns:
            Callable[[~.GetFindingRequest],
                    Awaitable[~.Finding]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_finding" not in self._stubs:
            self._stubs["get_finding"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/GetFinding",
                request_serializer=web_security_scanner.GetFindingRequest.serialize,
                response_deserializer=finding.Finding.deserialize,
            )
        return self._stubs["get_finding"]

    @property
    def list_findings(
        self,
    ) -> Callable[
        [web_security_scanner.ListFindingsRequest],
        Awaitable[web_security_scanner.ListFindingsResponse],
    ]:
        r"""Return a callable for the list findings method over gRPC.

        List Findings under a given ScanRun.

        Returns:
            Callable[[~.ListFindingsRequest],
                    Awaitable[~.ListFindingsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_findings" not in self._stubs:
            self._stubs["list_findings"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/ListFindings",
                request_serializer=web_security_scanner.ListFindingsRequest.serialize,
                response_deserializer=web_security_scanner.ListFindingsResponse.deserialize,
            )
        return self._stubs["list_findings"]

    @property
    def list_finding_type_stats(
        self,
    ) -> Callable[
        [web_security_scanner.ListFindingTypeStatsRequest],
        Awaitable[web_security_scanner.ListFindingTypeStatsResponse],
    ]:
        r"""Return a callable for the list finding type stats method over gRPC.

        List all FindingTypeStats under a given ScanRun.

        Returns:
            Callable[[~.ListFindingTypeStatsRequest],
                    Awaitable[~.ListFindingTypeStatsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_finding_type_stats" not in self._stubs:
            self._stubs["list_finding_type_stats"] = self.grpc_channel.unary_unary(
                "/google.cloud.websecurityscanner.v1beta.WebSecurityScanner/ListFindingTypeStats",
                request_serializer=web_security_scanner.ListFindingTypeStatsRequest.serialize,
                response_deserializer=web_security_scanner.ListFindingTypeStatsResponse.deserialize,
            )
        return self._stubs["list_finding_type_stats"]


__all__ = ("WebSecurityScannerGrpcAsyncIOTransport",)
